<?php
namespace Teknisa\CUS\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class Cus01200 extends \Zeedhi\Framework\Controller\Report {
    protected $dataSourceManager;
    protected $birtStrategy;
    protected $environment;
    protected $novoCodigo;
    protected $connection;

    public function __construct($crudService, $parameterBag, $dataSourceManager, $birtStrategy, $environment, $novoCodigo, $connection, $reportService) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->birtStrategy = $birtStrategy;
        $this->environment = $environment;
        $this->novoCodigo = $novoCodigo;
        $this->connection = $connection;
        $this->reportService = $reportService;
    }
    
    protected $reportMapping = array (
        'CUS01200' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                'CDTIPOCUSTO'    => 'P_NRTIPOCUSTO',   
                'PRODESTOC'      => 'P_IDESTOCAVEIS',  
                'CDFILIAL'       => 'P_CDFILIAL',
                'NRLISTA'        => 'P_NRLISTA',
                'DTINICIAL'      => 'P_DTINICIAL',     
                'DTFINAL'        => 'P_DTFINAL',       
                'CDARVPRODINI'   => 'P_CDPRODINIC',    
                'CDARVPRODFINAL' => 'P_CDPRODFINA',    
                'DTMESINIC'      => 'P_DTMESINIC',     
                'DTMESFINA'      => 'P_DTMESFINA',     
                'PRODNAOESTOC'   => 'P_IDNESTOCAVEIS', 
                'NMTIPOCUSTO'    => 'P_NMTIPOCUSTO',
                'CDPICTPROD'     => 'P_MASK',
                'NRORG'          => 'P_NRORG',
            ),
        )
    );

    public function createRemoteReport(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $this->beforeProcessReport($request, $response);
            $reportURL = $this->processReport($request);
            $response->addMessage(new DTO\Response\Message($reportURL));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        } 
    }
    
     public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $row["NRORG"] = $this->environment->getNrOrg();
        
        if($row["ALLCDSERVFILI"] == 'S'){
            $result =  self::setDataSourceServico($request,  $response, true);
            $cdServFili = "";
            
            foreach($result as $key => &$filial){
                $cdServFili.=$filial['CDSERVFILI']."_";
            }
            
            $row['NRLISTA'] = $this->saveLista($cdServFili);
        }else {
            $row['NRLISTA'] = $this->saveLista($this->concatParams($row, 'NMSERVICO'));
        }
        
        if (isset($row['QR2TYPE']) && $row['QR2TYPE']){
          $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        }
    }
    
    public function saveLista($row){
          $listafiliserv = explode("_",$row);
          $nrlista = $this->novoCodigo->getCodeByTableName('LISTA',$this->environment->getNrOrgTrab());
          $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
          foreach ($listafiliserv as $filiserv) {
                $stmt = $this->connection->prepare($sql);
                
                if(!empty($filiserv)){
                    $stmt->execute(
                        array(
                            "NRLISTA" => $nrlista,
                            "CDLISTA" => $filiserv
                        )
                    );     
                }
          }
          return $nrlista;
    }

    public function concatParams($row, $param){
        $concat = "";
        foreach($row[$param] as $value){
            $concat .= $value."_";
        }
        return $concat;
    }
    
    public function setDataSourceServico(DTO\Request\Row $request, DTO\Response $response, $flag = false){
        $row = $request->getRow();
        $param = array(
            "CDFILIAL"      => isset($row['CDFILIAL']) == false ? "" : $row['CDFILIAL'],
            "CDOPERADOR"    => isset($row['CDOPERADOR']) == false ? "" : $row['CDOPERADOR'],
            "CDGRUPSERV"    => isset($row['CDGRUPSERV']) == false ? "T" : $row['CDGRUPSERV'],
            "NRORG"         => isset($row['NRORG']) == false ? "" : $row['NRORG']
        );
        
        $query = "SELECT S.CDSERVFILI FROM SERVICO S, FILIOPER F WHERE S.CDFILIAL = :CDFILIAL AND((INSTR(:CDGRUPSERV, S.CDGRUPSERV) <> 0) OR (:CDGRUPSERV = 'T')) AND (F.CDOPERADOR = :CDOPERADOR) AND (F.CDFILIAL = S.CDFILIAL) AND (S.NRORG = :NRORG) ORDER BY NLSSORT(S.CDFILIAL,'NLS_SORT=BINARY_AI'), NLSSORT(S.CDSERVFILI,'NLS_SORT=BINARY_AI')";
        $resultServOper = $this->connection->fetchAll($query, $param);
        
        return $resultServOper;
    }
    
}