<?php
namespace Teknisa\CUS\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class Cus01500 extends \Zeedhi\Framework\Controller\Report { 

    protected $novoCodigo;
    protected $consplan;
    protected $dataSourceManager;
    protected $codigo;
    protected $environment;
    protected $birtStrategy;
    protected $connection;
    protected $reportService;

    public function __construct($crudService, $parameterBag, $consplan, $novoCodigo, $entityManager, $dataSourceManager, $environment, $birtStrategy, $connection, $reportService, $util) {
        $this->consplan          = $consplan;
        $this->novoCodigo        = $novoCodigo;
        $this->entityManager     = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment       = $environment;
        $this->crudService       = $crudService;
        $this->parameterBag      = $parameterBag;
        $this->birtStrategy      = $birtStrategy;
        $this->connection       = $connection;
        $this->reportService    = $reportService;
        $this->util             = $util;
    }


    protected $reportMapping = array (
        'CUS01500' => 
        array (
          'strategy' => 'BIRT',
          'parameterMapping' => 
          array (
            'CDTIPOCUSTO'       => 'P_NRTIPOCUSTO',
            'CDFILIAL'          => 'P_CDFILIAL',
            'TPDTCUS'           => 'P_TPDTCUS',
            'LISTASERV'         => 'P_CDSERVFILI',
            'NRCONSPLAN'        => 'P_NRCONSPLAN',
            'DTMESINIC'         => 'P_DTMESINIC',
            'DTMESFINA'         => 'P_DTMESFINA',
            'IDNESTOCAVEIS'     => 'P_IDNESTOCAVEIS',
            'DTINICIAL'         => 'P_DTINICIAL',
            'DTFINAL'           => 'P_DTFINAL',
            'CDARVPRODINI'      => 'P_CDPRODINIC',
            'CDARVPRODFINAL'    => 'P_CDPRODFINA',
            'NMTIPOCUSTO'       => 'P_NMTIPOCUSTO',
            'IDCONSFIXO'        => 'P_IDCONSFIXO',
            'IDCONSPERC'        => 'P_IDCONSPERC',
            'IDMATEPRIM'        => 'P_IDMATEPRIM',
            'MASK'              => 'P_MASK',
            'NRORG'             => 'P_NRORG'
          ),
        ),
    );

    public function createRemoteReport(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $this->beforeProcessReport($request, $response);
            $reportURL = $this->processReport($request);
            $response->addMessage(new DTO\Response\Message($reportURL));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        } 
    }
    
     public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $row["NRORG"] = $this->environment->getNrOrg();
        
        if (isset($row['QR2TYPE']) && $row['QR2TYPE']){
          $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        }

        $this->procedureConsumoPlanejado($row);
    }
    

    public function procedureConsumoPlanejado($row) {

        $dataFinal   = \DateTime::createFromFormat('d/m/Y', $row['DTFINAL']);
        $dataInicial = \DateTime::createFromFormat('d/m/Y', $row['DTINICIAL']);

        $tamanho = 10;
        $completaCom = 'z';
        $row['CDARVPRODFINAL'] = str_pad($row['CDARVPRODFINAL'], $tamanho, $completaCom, STR_PAD_RIGHT);
        
        if ($row['LISTASERV'] == 'REQUEST') {

            $this->dataSourceName = 'servicos_por_grupo';
            $this->parameterBagColumns = array('CDFILIAL', 'CDOPERADOR', 'CDGRUPSERV', 'NRORG');

            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('CDFILIAL', $row['CDFILIAL']);
            $filterCriteria->addCondition('CDOPERADOR', $row['CDOPERADOR']);
            $filterCriteria->addCondition('CDGRUPSERV', $row['LISTAGRUPSERV']);
            $filterCriteria->addCondition('NRORG', $row['NRORG']);

            $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
            
            $row['LISTASERV'] = $this->retornaServicosConcatenados($dataSet->getRows());
            if($row['LISTASERV'] == ""){
                $row['LISTASERV'] = "N";
            }
        }else if(!$row['LISTASERV'] == 'T'){
            $row['LISTASERV'] = $this->util->insertLista($row['LISTASERV']);
        }
        $nrConsPlan = $this->novoCodigo->getCodeByTableName('CONSPLAN'.$row['CDFILIAL'], $row['NRORG']);

        $connection = $this->entityManager->getConnection();
        $procedureConsumoPlanejado = new StoredProcedure($connection, 'CONSUMO_PLANEJADO_PLA');
        $procedureConsumoPlanejado->addParam(new Param('P_FILIAL',       Param::PARAM_INPUT, $row['CDFILIAL'],     Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_SERVICOS',     Param::PARAM_INPUT, $row['LISTASERV'],    Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['DTINICIAL'],    Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $row['DTFINAL'],      Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDARQUIVO',    Param::PARAM_INPUT, "T",                  Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, $row['IDPRODUTOS'],                Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, "N",                  Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, 0,                    Param::PARAM_TYPE_INT));
        $procedureConsumoPlanejado->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, "N",                  Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_INPUT, $nrConsPlan,              Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, $row['CDARVPRODINI'], Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, $row['CDARVPRODFINAL'], Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $row['CDOPERADOR'],   Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_STDTINICIAL',  Param::PARAM_INPUT, "_".$dataInicial->format('Ymd')."_", Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_STDTFINAL',    Param::PARAM_INPUT, "_".$dataFinal->format('Ymd')."_",   Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->execute();
        $this->params = array();

        $row['NRCONSPLAN']      = $nrConsPlan;

        // $servicoPlanejado = $this->curvaPorServicoPlanejado($row, $codigo);
        // $response->addDataset($servicoPlanejado);
        return $nrConsPlan;
    }

    public function curvaPorServicoPlanejado ($row, $codigo) {

        $this->dataSourceName = 'cus01500_abcservicoplan';
        $this->parameterBagColumns = array('P_NRCONSPLAN', 'P_CDFILIAL', 'P_NRTIPOCUSTO', 'P_TIPODATA', 'P_DTMESINIC', 'P_DTMESFINA', 'P_CDPRODINIC', 'P_CDPRODFINA', 'P_IDNESTOCAVEIS', 'P_SERVICOS', 'P_GRUPSERV');
        
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('P_NRCONSPLAN',       $codigo);
        $filterCriteria->addCondition('P_CDFILIAL',         $row['CDFILIAL']);
        $filterCriteria->addCondition('P_NRTIPOCUSTO',      $row['CDTIPOCUSTO']);
        $filterCriteria->addCondition('P_TIPODATA',         $row['TPDTCUS']);
        $filterCriteria->addCondition('P_DTMESINIC',        $row['DTMESINIC']);
        $filterCriteria->addCondition('P_DTMESFINA',        $row['DTMESFINA']);
        $filterCriteria->addCondition('P_CDPRODINIC',       $row['CDARVPRODINI']);
        $filterCriteria->addCondition('P_CDPRODFINA',       $row['CDARVPRODFINAL']);        
        $filterCriteria->addCondition('P_IDNESTOCAVEIS',    $row['IDNESTOCAVEIS']);
        $filterCriteria->addCondition('P_SERVICOS',         $row['LISTASERV']);
        $filterCriteria->addCondition('P_GRUPSERV',         $row['LISTAGRUPSERV']);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $dataSet;
    }

    public function retornaServicosConcatenados($servicos) {

        $newArray = array();

        foreach ($servicos as $servico) {

            $newArray[] = $servico['CDSERVFILI'];
        }

        return implode("_", $newArray);
    }

    // public function excluiConsplanCus01500(DTO\Request\Row $request, DTO\Response $response) {
    //     $parametros = array ("NRORG" => $this->environment->getNrOrg(), "CDFILICPLA" =>$this->environment->getCdFilial(), "NRCONSPLAN" => $this->codigo, "__is_new" => false);
    //     $newRequest = new DTO\Request\Row($parametros, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
    //     $newResponse = new DTO\Response();
    //     $this->consplan->delete($newRequest, $newResponse);
    // }    
}